@@ -18,6 +18,7 @@
 //usage:       "$ dumpkmap > keymap\n"
 
 #include "libbb.h"
+#include "common_bufsiz.h"
 
 /* From <linux/kd.h> */
 struct kbentry {
@@ -37,6 +38,7 @@ int dumpkmap_main(int argc UNUSED_PARAM, char **argv)
 	struct kbentry ke;
 	int i, j, fd;
 #define flags bb_common_bufsiz1
+	setup_common_bufsiz();
 
 	/* When user accidentally runs "dumpkmap FILE"
 	 * instead of "dumpkmap >FILE", we'd dump binary stuff to tty.
