@@ -9,11 +9,29 @@
  *
  * dos2unix filters reading input from stdin and writing output to stdout.
  *
- * Licensed under the GPL v2 or later, see the file LICENSE in this tarball.
+ * Licensed under GPLv2 or later, see file LICENSE in this source tree.
 */
 
+//usage:#define dos2unix_trivial_usage
+//usage:       "[-ud] [FILE]"
+//usage:#define dos2unix_full_usage "\n\n"
+//usage:       "Convert FILE in-place from DOS to Unix format.\n"
+//usage:       "When no file is given, use stdin/stdout.\n"
+//usage:     "\n	-u	dos2unix"
+//usage:     "\n	-d	unix2dos"
+//usage:
+//usage:#define unix2dos_trivial_usage
+//usage:       "[-ud] [FILE]"
+//usage:#define unix2dos_full_usage "\n\n"
+//usage:       "Convert FILE in-place from Unix to DOS format.\n"
+//usage:       "When no file is given, use stdin/stdout.\n"
+//usage:     "\n	-u	dos2unix"
+//usage:     "\n	-d	unix2dos"
+
 #include "libbb.h"
 
+/* This is a NOEXEC applet. Be very careful! */
+
 enum {
 	CT_UNIX2DOS = 1,
 	CT_DOS2UNIX
@@ -23,7 +41,7 @@ enum {
 static void convert(char *fn, int conv_type)
 {
 	FILE *in, *out;
-	int i;
+	int ch;
 	char *temp_fn = temp_fn; /* for compiler */
 	char *resolved_fn = resolved_fn;
 
@@ -31,30 +49,30 @@ static void convert(char *fn, int conv_type)
 	out = stdout;
 	if (fn != NULL) {
 		struct stat st;
+		int fd;
 
 		resolved_fn = xmalloc_follow_symlinks(fn);
 		if (resolved_fn == NULL)
 			bb_simple_perror_msg_and_die(fn);
 		in = xfopen_for_read(resolved_fn);
-		fstat(fileno(in), &st);
+		xfstat(fileno(in), &st, resolved_fn);
 
 		temp_fn = xasprintf("%sXXXXXX", resolved_fn);
-		i = mkstemp(temp_fn);
-		if (i == -1
-		 || fchmod(i, st.st_mode) == -1
-		) {
+		fd = xmkstemp(temp_fn);
+		if (fchmod(fd, st.st_mode) == -1)
 			bb_simple_perror_msg_and_die(temp_fn);
-		}
-		out = xfdopen_for_write(i);
+		fchown(fd, st.st_uid, st.st_gid);
+
+		out = xfdopen_for_write(fd);
 	}
 
-	while ((i = fgetc(in)) != EOF) {
-		if (i == '\r')
+	while ((ch = fgetc(in)) != EOF) {
+		if (ch == '\r')
 			continue;
-		if (i == '\n')
+		if (ch == '\n')
 			if (conv_type == CT_UNIX2DOS)
 				fputc('\r', out);
-		fputc(i, out);
+		fputc(ch, out);
 	}
 
 	if (fn != NULL) {
