@@ -9,6 +9,17 @@
 
 /* BB_AUDIT SUSv3 N/A -- Matches GNU behavior. */
 
+//config:config HOSTID
+//config:	bool "hostid"
+//config:	default y
+//config:	help
+//config:	  hostid prints the numeric identifier (in hexadecimal) for
+//config:	  the current host.
+
+//applet:IF_HOSTID(APPLET_NOFORK(hostid, hostid, BB_DIR_USR_BIN, BB_SUID_DROP, hostid))
+
+//kbuild:lib-$(CONFIG_HOSTID) += hostid.o
+
 //usage:#define hostid_trivial_usage
 //usage:       ""
 //usage:#define hostid_full_usage "\n\n"
@@ -25,7 +36,8 @@ int hostid_main(int argc UNUSED_PARAM, char **argv UNUSED_PARAM)
 		bb_show_usage();
 	}
 
-	printf("%lx\n", gethostid());
+	/* POSIX says gethostid returns a "32-bit identifier" */
+	printf("%08x\n", (unsigned)(uint32_t)gethostid());
 
 	return fflush_all();
 }
