@@ -5,7 +5,7 @@
  * Copyright (C) 2000 by Matt Kraai <kraai@alumni.carnegiemellon.edu>
  * SELinux support by Yuichi Nakamura <ynakam@hitachisoft.jp>
  *
- * Licensed under GPLv2 or later, see file LICENSE in this tarball for details.
+ * Licensed under GPLv2 or later, see file LICENSE in this source tree.
  */
 
 /* Mar 16, 2003      Manuel Novoa III   (mjn3@codepoet.org)
@@ -16,15 +16,34 @@
 #include "libbb.h"
 #include "libcoreutils/coreutils.h"
 
+//usage:#define mv_trivial_usage
+//usage:       "[-fin] SOURCE DEST\n"
+//usage:       "or: mv [-fin] SOURCE... DIRECTORY"
+//usage:#define mv_full_usage "\n\n"
+//usage:       "Rename SOURCE to DEST, or move SOURCE(s) to DIRECTORY\n"
+//usage:     "\n	-f	Don't prompt before overwriting"
+//usage:     "\n	-i	Interactive, prompt before overwrite"
+//usage:     "\n	-n	Don't overwrite an existing file"
+//usage:
+//usage:#define mv_example_usage
+//usage:       "$ mv /tmp/foo /bin/bar\n"
+
 #if ENABLE_FEATURE_MV_LONG_OPTIONS
 static const char mv_longopts[] ALIGN1 =
 	"interactive\0" No_argument "i"
 	"force\0"       No_argument "f"
+	"no-clobber\0"  No_argument "n"
+	IF_FEATURE_VERBOSE(
+	"verbose\0"     No_argument "v"
+	)
 	;
 #endif
 
-#define OPT_FILEUTILS_FORCE       1
-#define OPT_FILEUTILS_INTERACTIVE 2
+#define OPT_FORCE       (1 << 0)
+#define OPT_INTERACTIVE (1 << 1)
+#define OPT_NOCLOBBER   (1 << 2)
+#define OPT_VERBOSE     ((1 << 3) * ENABLE_FEATURE_VERBOSE)
+
 
 int mv_main(int argc, char **argv) MAIN_EXTERNALLY_VISIBLE;
 int mv_main(int argc, char **argv)
@@ -40,10 +59,12 @@ int mv_main(int argc, char **argv)
 #if ENABLE_FEATURE_MV_LONG_OPTIONS
 	applet_long_options = mv_longopts;
 #endif
-	// Need at least two arguments
-	// -f unsets -i, -i unsets -f
-	opt_complementary = "-2:f-i:i-f";
-	flags = getopt32(argv, "fi");
+	/* Need at least two arguments.
+	 * If more than one of -f, -i, -n is specified , only the final one
+	 * takes effect (it unsets previous options).
+	 */
+	opt_complementary = "-2:f-in:i-fn:n-fi";
+	flags = getopt32(argv, "finv");
 	argc -= optind;
 	argv += optind;
 	last = argv[argc - 1];
@@ -68,18 +89,22 @@ int mv_main(int argc, char **argv)
 		}
 
  DO_MOVE:
-		if (dest_exists
-		 && !(flags & OPT_FILEUTILS_FORCE)
-		 && ((access(dest, W_OK) < 0 && isatty(0))
-		    || (flags & OPT_FILEUTILS_INTERACTIVE))
-		) {
-			if (fprintf(stderr, "mv: overwrite '%s'? ", dest) < 0) {
-				goto RET_1;	/* Ouch! fprintf failed! */
-			}
-			if (!bb_ask_confirmation()) {
+		if (dest_exists) {
+			if (flags & OPT_NOCLOBBER)
 				goto RET_0;
+			if (!(flags & OPT_FORCE)
+			 && ((access(dest, W_OK) < 0 && isatty(0))
+			    || (flags & OPT_INTERACTIVE))
+			) {
+				if (fprintf(stderr, "mv: overwrite '%s'? ", dest) < 0) {
+					goto RET_1;  /* Ouch! fprintf failed! */
+				}
+				if (!bb_ask_confirmation()) {
+					goto RET_0;
+				}
 			}
 		}
+
 		if (rename(*argv, dest) < 0) {
 			struct stat source_stat;
 			int source_exists;
@@ -87,10 +112,10 @@ int mv_main(int argc, char **argv)
 			if (errno != EXDEV
 			 || (source_exists = cp_mv_stat2(*argv, &source_stat, lstat)) < 1
 			) {
-				bb_perror_msg("cannot rename '%s'", *argv);
+				bb_perror_msg("can't rename '%s'", *argv);
 			} else {
 				static const char fmt[] ALIGN1 =
-					"cannot overwrite %sdirectory with %sdirectory";
+					"can't overwrite %sdirectory with %sdirectory";
 
 				if (dest_exists) {
 					if (dest_exists == 3) {
@@ -105,7 +130,7 @@ int mv_main(int argc, char **argv)
 						}
 					}
 					if (unlink(dest) < 0) {
-						bb_perror_msg("cannot remove '%s'", dest);
+						bb_perror_msg("can't remove '%s'", dest);
 						goto RET_1;
 					}
 				}
@@ -126,6 +151,9 @@ int mv_main(int argc, char **argv)
 			status = 1;
 		}
  RET_0:
+		if (flags & OPT_VERBOSE) {
+			printf("'%s' -> '%s'\n", *argv, dest);
+		}
 		if (dest != last) {
 			free((void *) dest);
 		}
