@@ -32,15 +32,14 @@
 //usage:       "Print last 10 lines of each FILE (or stdin) to stdout.\n"
 //usage:       "With more than one FILE, precede each with a filename header.\n"
 //usage:     "\n	-f		Print data as file grows"
-//usage:	IF_FEATURE_FANCY_TAIL(
-//usage:     "\n	-s SECONDS	Wait SECONDS between reads with -f"
-//usage:	)
+//usage:     "\n	-c [+]N[kbm]	Print last N bytes"
 //usage:     "\n	-n N[kbm]	Print last N lines"
 //usage:     "\n	-n +N[kbm]	Start on Nth line and print the rest"
 //usage:	IF_FEATURE_FANCY_TAIL(
-//usage:     "\n	-c [+]N[kbm]	Print last N bytes"
 //usage:     "\n	-q		Never print headers"
+//usage:     "\n	-s SECONDS	Wait SECONDS between reads with -f"
 //usage:     "\n	-v		Always print headers"
+//usage:     "\n	-F		Same as -f, but keep retrying"
 //usage:     "\n"
 //usage:     "\nN may be suffixed by k (x1024), b (x512), or m (x1024^2)."
 //usage:	)
@@ -50,13 +49,14 @@
 //usage:       "nameserver 10.0.0.1\n"
 
 #include "libbb.h"
+#include "common_bufsiz.h"
 
 struct globals {
 	bool from_top;
 	bool exitcode;
 } FIX_ALIASING;
-#define G (*(struct globals*)&bb_common_bufsiz1)
-#define INIT_G() do { } while (0)
+#define G (*(struct globals*)bb_common_bufsiz1)
+#define INIT_G() do { setup_common_bufsiz(); } while (0)
 
 static void tail_xprint_header(const char *fmt, const char *filename)
 {
