@@ -481,9 +481,13 @@ typedef unsigned smalluint;
 
 #if defined(ANDROID) || defined(__ANDROID__)
 # if __ANDROID_API__ < 8
+   /* ANDROID < 8 has no [f]dprintf at all */
 #  undef HAVE_DPRINTF
-# else
+# elif __ANDROID_API__ < 21
+   /* ANDROID < 21 has fdprintf */
 #  define dprintf fdprintf
+# else
+   /* ANDROID >= 21 has standard dprintf */
 # endif
 # if __ANDROID_API__ < 21
 #  undef HAVE_TTYNAME_R
