@@ -7,15 +7,16 @@
  *
  * Returns 1 for a match, otherwise 0
  *
- * Licensed under GPLv2 or later, see file LICENSE in this tarball for details.
+ * Licensed under GPLv2 or later, see file LICENSE in this source tree.
  */
 
 #include "libbb.h"
 
+#ifdef HAVE_MNTENT_H
+
 int FAST_FUNC match_fstype(const struct mntent *mt, const char *t_fstype)
 {
 	int match = 1;
-	int len;
 
 	if (!t_fstype)
 		return match;
@@ -25,10 +26,10 @@ int FAST_FUNC match_fstype(const struct mntent *mt, const char *t_fstype)
 		t_fstype += 2;
 	}
 
-	len = strlen(mt->mnt_type);
 	while (1) {
-		if (strncmp(mt->mnt_type, t_fstype, len) == 0
-		 && (t_fstype[len] == '\0' || t_fstype[len] == ',')
+		char *after_mnt_type = is_prefixed_with(t_fstype, mt->mnt_type);
+		if (after_mnt_type
+		 && (*after_mnt_type == '\0' || *after_mnt_type == ',')
 		) {
 			return match;
 		}
@@ -40,3 +41,5 @@ int FAST_FUNC match_fstype(const struct mntent *mt, const char *t_fstype)
 
 	return !match;
 }
+
+#endif /* HAVE_MNTENT_H */
